@@ -1,5 +1,5 @@
 /* GMM (C) 2018-2019 Stephane Charette <stephanecharette@gmail.com>
- * $Id: TabCanvas.cpp 2718 2019-02-07 07:08:42Z stephane $
+ * $Id: TabCanvas.cpp 2719 2019-02-08 06:25:55Z stephane $
  */
 
 #include "GMM.hpp"
@@ -194,15 +194,22 @@ void TabCanvas::timerCallback(void)
 
 	if (threadShouldExit() == false)
 	{
-		periodic_update();
+		try
+		{
+			periodic_update();
 
-		// There are 2 different ways we can update the kiln/run/package numbers.  The first is with the OPC Console,
-		// and the second is with the automatic .csv file that Curt wrote in VB.  The .csv solution was temporary and
-		// the OPC Console method is the prefered method.  The code still attempts both, and returns if the .csv or
-		// OPC Console output file does not exist on disk.
+			// There are 2 different ways we can update the kiln/run/package numbers.  The first is with the OPC Console,
+			// and the second is with the automatic .csv file that Curt wrote in VB.  The .csv solution was temporary and
+			// the OPC Console method is the prefered method.  The code still attempts both, and returns if the .csv or
+			// OPC Console output file does not exist on disk.
 
-		parse_opc_console_output();
-		read_opc_values_from_csv();
+			parse_opc_console_output();
+			read_opc_values_from_csv();
+		}
+		catch (const std::exception &e)
+		{
+			LOG_MSG("prefix=" << prefix << ": ignoring the exception caught during timer callback: " << e.what());
+		}
 
 		if (timer_frequency_in_milliseconds > 0)
 		{
@@ -980,7 +987,7 @@ void TabCanvas::read_all_mc_files(void)
 
 		if (need_to_clear_out_old_package_values)
 		{
-			LOG_MSG("prefix=" << prefix << ": clearing out all package buckets and " << pkg_doubles.size() << " board entries to load k=" << current_kiln << " << r=" << current_run << " p=" << package_index << " from " << filename_ss.str() << " (run so far contains " << run_doubles.size() << " boards)");
+			LOG_MSG("prefix=" << prefix << ": clearing out all package buckets and " << pkg_doubles.size() << " board entries to load k=" << current_kiln << " r=" << current_run << " p=" << package_index << " from " << filename_ss.str() << " (run so far contains " << run_doubles.size() << " boards)");
 
 			// we're starting to read a new package, so clear out the package-specific values
 			for (size_t idx = 0; idx < number_of_buckets; idx++)
@@ -1111,8 +1118,27 @@ void TabCanvas::parse_opc_console_output(void)
 	 * The order of the lines is not guaranteed.
 	 */
 
+	size_t new_sequence_number = 0;
+
 	// first line is the sequence number
-	const size_t new_sequence_number = std::stoul(v[0]);
+	try
+	{
+		new_sequence_number = std::stoul(v[0]);
+	}
+	catch (...)
+	{
+		// I've seen this throw in Gorman's log file, presumably because OPC was in the middle of writing out a new output file
+		LOG_MSG("prefix=" << prefix << ": failed to parse the sequence number (\"" << v[0] << "\") from the OPC output file \"" << opc_output_filename << "\"");
+		return;
+	}
+	if (new_sequence_number < 1)
+	{
+		// invalid sequence number -- ignore the file for now, we'll catch it again
+		// in 1 second when this function is called again once the timer fires
+		LOG_MSG("prefix=" << prefix << ": failed to obtain a valid sequence number (\"" << v[0] << "\" -> " << new_sequence_number << ") from the OPC output file \"" << opc_output_filename << "\"");
+		return;
+	}
+
 	if (new_sequence_number == opc_sequence_number)
 	{
 		// file hasn't changed, so nothing else we need to do
